/**
 * Prakticka cast diplomove prace
 * Reprezentace textu a její vliv na kategorizaci
 *
 * Projekt: Program pro predzpracovani textovych dat
 *            - nacita textova data nebo dataset Reuters-21578
 *            - umoznuje nastavit metody predzpracovani, vybrat model
 *              a rysy reprezentace
 *            - zameruje se take na reprezentaci dokumentu pomoci N-Gramu
 *            - uklada data ve formatu XML nebo CSV
 * Autor:   Bc. Ondrej Sabatka
 * Kontakt: xsabat05@stud.fit.vutbr.cz
 * Datum:   unor - duben 2010
 */

package tp.controllers;

import java.awt.Cursor;
import java.io.File;
import javax.swing.SwingWorker;
import tp.model.DocumentsDatabase;
import tp.model.LinkToFiles;
import tp.loader.LoaderInterface;
import tp.loader.LoaderPlainText;
import tp.loader.LoaderReuters;
import tp.view.ProgressView;
import tp.view.TabLoaderView;
import tp.view.TabView;

/**
 * A controller to process events on a screen, where the inputs are selected
 * @author Ondrej Sabatka
 */
public class LoaderControl extends ActionControl {

    //Clenske promenne tridy
    private LinkToFiles link_files;
    private TaskLoad task;
    private LoaderInterface loader;
    private DocumentsDatabase database;

    /**
     * Constructor of the class
     */
    public LoaderControl() {}



    /**
     * If the "Next" button is clicked, this method creates a list of files to be
     * processed a passes it to the controller of pre-processing.
     */
    public void nextBtnClick() {
        
        //If no files are selected, it is not possible to continue
        if((link_files == null) || link_files.isEmpty()) {
            MainControl.getInstance().getMainView().
                        showErrorDialog("No documents selected to process!");

            //The method is stopped
            return;
        }

        //Setting od the input file format chosen by the user
        TabView tab_view = MainControl.getInstance().getMainView().getTabView();
        int file_format = tab_view.getTabLoader().getFileFormat();

        //A loader for reading data from files to the memory is created.
        loader = null;

        switch(file_format) {
            case TabLoaderView.PLAIN_TEXT:
                 link_files.setFileType(LinkToFiles.FileType.PLAIN_TEXT);
                 //Initialization of loader (according to the format)
                 loader = new LoaderPlainText();
                 break;
            case TabLoaderView.REUTERS:
                 link_files.setFileType(LinkToFiles.FileType.REUTERS);
                 loader = new LoaderReuters();
                 break;
        }


        //If a format is chosen, a file is read
        if(loader != null) {
            //Reading runs in the background and the progress can be viewed
            //on the progress bar to be sure that program is running
            task = new TaskLoad();
            task.execute();
        }
    }


    /**
     * A class, which ensures execution of file reading process.
     */
    class TaskLoad extends SwingWorker<Void, Void> {

        /**
         * A method which initailizes and executes file reading in the background.
         */
        @Override
        public Void doInBackground() {
            //Cursor is set to wait and progress bar is initialized
            MainControl.getInstance().getMainView().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
            ProgressView progress = MainControl.getInstance().getMainView().getProgressBar();
            progress.start("Reading files");
            
            loader.addPropertyChangeListener(progress);

            //A method to read the files is called
            database = loader.loadFromFiles(link_files);

            return null;
        }


        /**
         * A method, which is executed after file reading is finished.
         */
        @Override
        public void done() {
            if(database != null) {
                MainControl.getInstance().gotoSettings(database);
            }
            else {
                MainControl.getInstance().getMainView().
                        showErrorDialog("Reading of data failed!");
            }

            //Cursor is set to default and progress bar is stopped
            MainControl.getInstance().getMainView().setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
            MainControl.getInstance().getMainView().getProgressBar().stop();
        }
    }


    /**
     * After clict on "Load" button, an Open dialog is opened and then the chosen files
     * are inserted into a list.
     */
    public void loadBtnClick() {
        //Reference to view instance
        TabLoaderView tab_view = MainControl.getInstance().getMainView().getTabView().getTabLoader();

        //Array of files initialization
        if(link_files == null) {
            link_files = new LinkToFiles();
        }

        //Get files from user...
        File[] files_from_user = tab_view.getFilesFromUser();

        //If Cancel was not presses, file names are stored into the model and the actual
        //list of files is shown.
        if(files_from_user != null) {
            //Storage of chosen files
            int flag = link_files.addFiles(files_from_user);

            //Array of file names
            String[] files_name = new String[link_files.getCount()];

            for(int i = 0; i < link_files.getCount(); i++) {
                String parrent_addr = link_files.getFileAt(i).getParent();

                File dir_path = link_files.getFileAt(i).getParentFile();
                String redundant = dir_path.getParent() + File.separator;

                parrent_addr = parrent_addr.substring(redundant.length(), parrent_addr.length());
                files_name[i] = File.separator + parrent_addr + File.separator + link_files.getFileAt(i).getName();
            }

            //A list of file names is shown
            tab_view.fillFileList(files_name);

            //Checks if sll files have been read
            if(flag != 0) {
                MainControl.getInstance().getMainView().
                        showErrorDialog("Some of selected files have not already been read!");
            }
        }
    }


    /**
     * After click on "Delete" button, a method deletes chosen files from the list.
     */
    public void deleteBtnClick() {
        //Reference to view instance
        TabLoaderView tab_view = MainControl.getInstance().getMainView().getTabView().getTabLoader();

        //Determines, which files were chosen to delete
        link_files.removeFiles(tab_view.getSelectedFiles());

        //Array with file names...
        String[] files_name = new String[link_files.getCount()];

        for(int i = 0; i < link_files.getCount(); i++) {
            String parrent_addr = link_files.getFileAt(i).getParent();

            File dir_path = link_files.getFileAt(i).getParentFile();
            String redundant = dir_path.getParent() + File.separator;

            parrent_addr = parrent_addr.replaceAll(redundant, "");
            files_name[i] = File.separator + parrent_addr + File.separator + link_files.getFileAt(i).getName();
        }

        //Modified list with file names is shown
        tab_view.fillFileList(files_name);
    }


    /**
     * After click on "Clear" button, all fine names are deleted from the list.
     */
    public void clearBtnClick() {
        link_files = null;
        MainControl.getInstance().getMainView().getTabView().getTabLoader().clearFileList();
    }
    
}
