/**
 * Prakticka cast diplomove prace
 * Reprezentace textu a její vliv na kategorizaci
 *
 * Projekt: Program pro predzpracovani textovych dat
 *            - nacita textova data nebo dataset Reuters-21578
 *            - umoznuje nastavit metody predzpracovani, vybrat model
 *              a rysy reprezentace
 *            - zameruje se take na reprezentaci dokumentu pomoci N-Gramu
 *            - uklada data ve formatu XML nebo CSV
 * Autor:   Bc. Ondrej Sabatka
 * Kontakt: xsabat05@stud.fit.vutbr.cz
 * Datum:   unor - duben 2010
 */

package tp.controllers;

import tp.model.DocumentsDatabase;
import tp.model.DocsRepresentDB;
import tp.model.ResourcesModel;
import tp.view.MainView;
import tp.view.TabView;

/**
 * Main controller which executes component controllers and actions
 * (Singleton design pattern used)
 * @author Ondrej Sabatka
 */

public class MainControl {
    //Atatic obect with class instance - if not initialized, contains NULL
    private static MainControl instance;
    private MainView main_window;
    private MenuControl main_menu_controller;
    private LoaderControl loader_controller;
    private ResourcesModel resource_model;
    private HelpControl help_controller;
    private SettingsControl settings_controller;
    private ResultsControl result_controller;
    private SaveControl save_controller;

    //Constructor
    protected MainControl(){}


    /**
     * Method used to create a Singleton
     * @return instance of the singleton
     */
    public static MainControl getInstance() {
       //If the instance is equal to NULL, a new object is created, otherwise instance is returned.
        if (instance == null) {
            instance = new MainControl();
        }

        return instance;
    }


    /**
     * The method initializes the main window of the application and initializes component
     * controllers.
     */
    public void startAppliacation() {
        //initialization of component controllers
        main_menu_controller = new MenuControl();
        help_controller = new HelpControl();

        //initialization of model
        resource_model = new ResourcesModel();

        loader_controller = new LoaderControl();
        settings_controller = new SettingsControl();
        result_controller = new ResultsControl();
        save_controller = new SaveControl();


        //Execution of the application - opening of the main window.
        main_window = new MainView();
    }


    /**
     * Method is used to change from data reading to pre-processing and representation setting.
     * It initializes controller for settings and gives it reference to a database 
     * of files.
     * An object for data reading is destroyed.
     * @param database reference to a database of files
     */
    public void gotoSettings(DocumentsDatabase database) {
        //Initialization of controller for settings
        settings_controller.init(database);

        //Showing the tab for settings (if file reading is OK)
        main_window.getTabView().showTab(TabView.SETTINGS);

        loader_controller = null;
    }

    /**
     * Method is used to change from pre-processing and representation setting to its results.
     * It initializes controller for results and gives it reference to a database
     * of document representation.
     * An object for pre-processing and representation setting tab is destroyed.
     * @param database reference to a database of document representations
     */
    public void gotoResults(DocsRepresentDB database) {
        //Initialization of controller for results
        result_controller.init(database);

        //Showing the tab for results (if pre-processing is OK)
        main_window.getTabView().showTab(TabView.RESULTS);

        settings_controller = null;
    }

    /**
     * Method is used to change from pre-processing resuts to save the results tab.
     * It initializes controller for saving and gives it reference to a database
     * of document representation.
     * An object for results tab is destroyed.
     * @param database reference to a database of document representations
     */
    public void gotoSave(DocsRepresentDB database) {
        //Initialization of controller for results saving
        save_controller.init(database);

        //Showing the tab for results saving (if everything is OK)
        main_window.getTabView().showTab(TabView.SAVE);

        result_controller = null;
    }


    /**
     * A function, which returns the istance of the main window.
     * @return instance of MainView
     */
    public MainView getMainView() {
        return main_window;
    }


    /**
     * A function, which returns the controller for application menu
     * @return instance of MainMenuControl
     */
    public MenuControl getMainMenuController() {
        return main_menu_controller;
    }

    
    /**
     * A function, which returns the reference to class which handles resources of application
     * @return instance of ResourceModel
     */
    public ResourcesModel getResourceModel() {
        return resource_model;
    }


    /**
     * A function, which returns the reference to controller for help panel
     * @return instance HelpControl
     */
    public HelpControl getHelpControl() {
        return help_controller;
    }


    /**
     * A function, which returns the reference to controller for loader
     * @return instance of LoaderControl
     */
    public LoaderControl getLoaderControl() {
        return loader_controller;
    }


    /**
     * A function, which returns the reference to controller for preprocessing settings
     * @return instance of SettingsControl
     */
    public SettingsControl getSettingsControl() {
        return settings_controller;
    }

    /**
     * A function, which returns the reference to controller for results
     * @return instance of ResultsControl
     */
    public ResultsControl getResultsControl() {
        return result_controller;
    }

    /**
     * A function, which returns the reference to controller for saving the results
     * @return instance of SaveControl
     */
    public SaveControl getSaveControl() {
        return save_controller;
    }
}
