/**
 * Prakticka cast diplomove prace
 * Reprezentace textu a její vliv na kategorizaci
 *
 * Projekt: Program pro predzpracovani textovych dat
 *            - nacita textova data nebo dataset Reuters-21578
 *            - umoznuje nastavit metody predzpracovani, vybrat model
 *              a rysy reprezentace
 *            - zameruje se take na reprezentaci dokumentu pomoci N-Gramu
 *            - uklada data ve formatu XML nebo CSV
 * Autor:   Bc. Ondrej Sabatka
 * Kontakt: xsabat05@stud.fit.vutbr.cz
 * Datum:   unor - duben 2010
 */
/* Cast metod je prevzata z diplomoveho projektu Ing. Martina Uhlire str nazvem
 * Metody pro ziskavani asociacnich pravidel z dat, Brno 2007.
 * Metoda byla modifikovana a upravena, aby vyhovovala pozadavkum tohoto
 * projektu. */

package tp.loader;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Stack;
import java.util.StringTokenizer;
import javax.swing.SwingWorker;
import tp.controllers.MainControl;
import tp.model.Document;
import tp.model.DocumentsDatabase;
import tp.model.LinkToFiles;

/**
 * Class for loading the text files in the Reuters dataset format.
 * @author Ondrej Sabatka
 */

public class LoaderReuters extends SwingWorker<Void, Void> implements LoaderInterface {

    //Constructor
    public LoaderReuters() {}


    /**
     * Method, which processes files from Reuters dataset and stores its contents
     * into a database of documents.
     * a jejich obsah uklada do databaze dokumentu.
     * @param link_files links to files for loading
     * @return reference to the database of documents
     */
    public DocumentsDatabase loadFromFiles(LinkToFiles link_files) {
        DocumentsDatabase database = new DocumentsDatabase();
        DataCleaner cleaner = new DataCleaner();
        ArrayList<DocumentContentTopic> documents = null;


        //Reads SGML files and loads all contained documents
        for(int i = 0; i < link_files.getCount(); i++) {
            //Loads documents from 1st SGML file
            documents = loadFromSGML(link_files.getFileAt(i));

            //Error
            if(documents == null) {
                return null;
            }

            //All loaded documents are stored...
            for(int j = 0; j < documents.size(); j++) {
                //Cleaning document contents
                String content_file = cleaner.cleanContentFile(documents.get(j).getContent());

                //Add a document to database of all documents
                database.addDocument(new Document(content_file, documents.get(j).getTopic()));
            }

            setProgress(i * 100 / link_files.getCount());
        }

        return database;
    }


    /**
     * Class for transfer of document contents and topic
     */
    private class DocumentContentTopic {

        //Variables
        private String content;
        private String topic;

        public DocumentContentTopic(String content, String topic) {
            this.content = content;
            this.topic = topic;
        }

        public String getContent() {
            return content;
        }

        public String getTopic() {
            return topic;
        }
        
    }


    /**
     * Method for processing of one SGML file from Reuters dataset, which returns
     * array of strings, each of which contains words from document.
     * @param f SGML file
     * @return array of text documents
     */
    private ArrayList<DocumentContentTopic> loadFromSGML(File f) {
        BufferedReader fin = null;
        String line, title_document = new String(), content_document = new String(), topic = new String();
        ArrayList<DocumentContentTopic> content_documents = new ArrayList<DocumentContentTopic>();
        DataCleaner cleaner = new DataCleaner();

        // string from input
        String string = new String();

        Stack<String> stackOfMarks = new Stack<String>();


        try {
            fin = new BufferedReader(new FileReader(f.getAbsoluteFile()));

            // parse one row
            line = null;
            boolean documentBrief = false;

            // scanning the file row by row
            while ((line = fin.readLine()) != null) {
                // st = parsing a row
                StringTokenizer st = new StringTokenizer(line);
                while (st.hasMoreTokens()) {
                    // tokenizing
                    string = st.nextToken();

                    int indexOfBegin = 0;
                    int indexOfEnd = 0;
                    // word between <TITLE> and </TITLE> or
                    // <BODY> and </BODY>
                    String word = "";

                    // if it has only <TITLE> and </TITLE>, but not <BODY></BODY>,
                    // a number is added without document to hold nubering
                    if ((line.contains("<TEXT TYPE=\"BRIEF\""))
                            && (documentBrief == false)) {
                        documentBrief = true;
                    }

                    if (string.contains("<TOPICS>")) {
                        String beginMark = "<TOPICS>";
                        stackOfMarks.push(beginMark);
                        indexOfBegin = string.indexOf(beginMark)
                                        + beginMark.length();

                    } else if (string.contains("<TITLE>")) {
                        String beginMark = "<TITLE>";
                        stackOfMarks.push(beginMark);
                        indexOfBegin = string.indexOf(beginMark)
                                        + beginMark.length();

                    } else if (string.contains("<BODY>")) {
                        String beginMark = "<BODY>";
                        stackOfMarks.push(beginMark);
                        indexOfBegin = string.indexOf(beginMark)
                                        + beginMark.length();
                        
                    } else {
                        indexOfBegin = 0;
                    }




                    if (string.contains("</TOPICS>")) {
                        String endMark = "</TOPICS>";

                        //if a mark TOPIC is missing, file is damaged -> error
                        if (!stackOfMarks.peek().equals("<TOPICS>")) {
                                MainControl.getInstance().getMainView().
                                    showErrorDialog("SGML file is damaged!\n(missing opening or closing tag)");
                                return null;
                        }

                        // pop <TOPICS> from stack
                        stackOfMarks.pop();
                        indexOfEnd = string.indexOf(endMark);
                        // pushing to the output indexes between
                        // indexOfBegin and indexOfEnd
                        topic = new String(string.substring(indexOfBegin,
                                        indexOfEnd));

                        // topic cleaning
                        topic = cleaner.cleanTopic(topic);

                        
                        ////////////////////////////////////////////////////////
                        //                  TOPIC RESTRICTIONS                //
                        ////////////////////////////////////////////////////////
                        //Some articles have more TOPICs; this is not supported here
                        // the first one is taken as the most relevant

                        StringTokenizer st_topic = new StringTokenizer(topic);

                        if(st_topic.hasMoreTokens()) {
                            topic = st_topic.nextToken();
                        }

                        
                    } else if (string.contains("</TITLE>")) {
                        String endMark = "</TITLE>";

                        //If opening TITLE tag is missing, file is damaged -> error
                        if (!stackOfMarks.peek().equals("<TITLE>")) {
                                MainControl.getInstance().getMainView().
                                    showErrorDialog("SGML file is damaged!\n(missing opening or closing tag))");
                                return null;
                        }

                        // pop <TITLE> from stack
                        stackOfMarks.pop();
                        indexOfEnd = string.indexOf(endMark);
                        // pushing to the output indexes between
                        // indexOfBegin and indexOfEnd
                        word = new String(string.substring(indexOfBegin,
                                        indexOfEnd));
                        word = cleaner.cleanSGMLReuters(word);

                        //Title of document is stored
                        title_document = title_document + " " + word;
                        title_document = title_document.trim();


                        //If document is brief, reading is finished and it is stored
                        if(documentBrief) {
                            content_documents.add(new DocumentContentTopic(title_document, topic));
                            title_document = new String();
                            topic = new String();

                            documentBrief = false;
                        }


                    } else if (string.contains("</BODY>")) {
                        String endMark = "</BODY>";
                        // Error in tags - end of BODY, but not opened
                        if (!stackOfMarks.peek().equals("<BODY>")) {
                                MainControl.getInstance().getMainView().
                                    showErrorDialog("SGML file is damaged!\n(missing opening or closing tag)");
                                return null;
                        }

                        // pop <BODY> from stack
                        stackOfMarks.pop();
                        indexOfEnd = string.indexOf(endMark);
                        // pushing to the output indexes between
                        // indexOfBegin and indexOfEnd
                        word = new String(string.substring(indexOfBegin,
                                        indexOfEnd));
                        word = cleaner.cleanSGMLReuters(word);

                        //Document contents is stored
                        content_document = content_document + " " + word;
                        content_document = content_document.trim();

                        //After </BODY> is loaded, title and contents is stored into
                        //array of all documents
                        
                        content_documents.add(
                                new DocumentContentTopic(title_document + ". " + content_document, topic));

                        //Initialization of title and contents of the next document
                        content_document = new String();
                        title_document = new String();
                        topic = new String();
                        

                    } else {
                        indexOfEnd = string.length();
                    }

                    //Opening tag on stack -> words are added to contents or title
                    if (!stackOfMarks.empty()) {
                        if (stackOfMarks.peek().equals("<TITLE>")) {
                            word = new String(string.substring(indexOfBegin,
                                            indexOfEnd));
                            word = cleaner.cleanSGMLReuters(word);
                            
                            title_document = title_document + " " + word;
                            title_document = title_document.trim();

                        } else if (stackOfMarks.peek().equals("<BODY>")) {
                            word = new String(string.substring(indexOfBegin,
                                            indexOfEnd));
                            word = cleaner.cleanSGMLReuters(word);
                            
                            content_document = content_document + " " + word;
                            content_document = content_document.trim();
                        }

                    } // if (!stackOfMarks.empty())

                } // while (st.hasMoreTokens())

            } // while ((line = input.readLine()) != null)

        } catch (FileNotFoundException ex) {
            MainControl.getInstance().getMainView().
                        showErrorDialog("File not found!");
            return null;
        } catch (IOException ex) {
            MainControl.getInstance().getMainView().
                        showErrorDialog("Error in reading file!");
            return null;
        } catch (Exception ex) {
            MainControl.getInstance().getMainView().
                        showErrorDialog("Error in file processing!");
            return null;
        } finally {
            try {
                if (fin != null) {
                    //Finish reading
                    fin.close();
                }
            } catch (IOException ex) {
                MainControl.getInstance().getMainView().
                            showErrorDialog("Error in file processing!");
                return null;
            }
        }

        //Resulting array
        return content_documents;
    }

    @Override
    protected Void doInBackground() throws Exception {
        throw new UnsupportedOperationException("Not supported yet.");
    }

}
