/**
 * Prakticka cast diplomove prace
 * Reprezentace textu a její vliv na kategorizaci
 *
 * Projekt: Program pro predzpracovani textovych dat
 *            - nacita textova data nebo dataset Reuters-21578
 *            - umoznuje nastavit metody predzpracovani, vybrat model
 *              a rysy reprezentace
 *            - zameruje se take na reprezentaci dokumentu pomoci N-Gramu
 *            - uklada data ve formatu XML nebo CSV
 * Autor:   Bc. Ondrej Sabatka
 * Kontakt: xsabat05@stud.fit.vutbr.cz
 * Datum:   unor - duben 2010
 */

package tp.model;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Vector;

/**
 * Class representing all documents represented by the DocRepresentTable class.
 * @author Ondrej Sabatka
 */

public class DocsRepresentDB {

    //Variables
    private Vector<DocRepresentTable> documents;

    //cursor
    private int cursor;
    private static final int NOT_INIT = -1;
    private static final int START = 0;

    //Pre-processing options
    public enum RepresentationModel {BINARY, TF, TFIDF}
    public enum RepresentationFeature {WORD, NGRAMS}
    public enum Preprocessing {NONE, STOP_LIST, STEMMING, STOP_LIST_STEMMING}

    private RepresentationModel model;
    private RepresentationFeature feature;
    private Preprocessing preprocessing;

    //N-Grams options
    private int ngram_depth = 0;
    private int min_ngram_occur = 0;


    /**
     * Class to store frequencies of features in documents.
     */
    public class FeatureDF implements Comparable {
        public int document_frequent;
        public String feature;

    /**
     * Class construtor.
     * @param feature a word
     * @param document_frequent document frequency of a word
     */
        public FeatureDF(String feature, int document_frequent) {
            this.document_frequent = document_frequent;
            this.feature = feature;
        }

    /**
     * Comparing two FeatureDF objects
     * @param t the second object
     */
        public int compareTo(Object t) {
            FeatureDF test = (FeatureDF) t;

            if(this.document_frequent == test.document_frequent) {
                return 0;
            }
            else if((this.document_frequent < test.document_frequent)) {
                return -1;
            }
            else {
                return 1;
            }
        }
    }

    //Hash table to compute feature frequencies in documents.
    private Hashtable<String, FeatureDF> features_document_frequency;

    private static final int FIRST_PRESENCE = 1;


    //*************** VARIABLES FOR STATISTICS ***************//
    private int count_features;
    private int max_count_features;  //v dokumentu
    private double avg_count_features;

    private int max_frequent_feature;
    private double avg_frequent_feature;
    private double max_tf_feature;
    private double avg_tf_feature;
    private double max_tfidf_feature;
    private double avg_tfidf_feature;
    



    /**
     * Class construtor.
     * @param model a model for document representation
     * @param feature tyoe of feature used in this representation
     * @param preprocessing pre-processing options
     */
    public DocsRepresentDB(RepresentationModel model, RepresentationFeature feature, Preprocessing preprocessing) {
        this.model = model;
        this.feature = feature;
        this.preprocessing = preprocessing;

        //Variable initialization for computing occurencies
        this.features_document_frequency = new Hashtable<String, FeatureDF>();

        this.documents = new Vector<DocRepresentTable>();
        this.cursor = NOT_INIT;
    }


    /**
     * Adds a new document representation.
     * @param document document to be added
     */
    public void addDocument(DocRepresentTable document) {
        //Actualization of occurence in all documents.
        includeFeaturesToCounter(document);

        this.documents.add(document);
    }


    /**
     * Method inserts all features from a document into a collective representation,
     * which is used to count TF-IDF
     * @param document one document representation
     */
    private void includeFeaturesToCounter(DocRepresentTable document) {
        ArrayList<FeatureInterface> features_array = document.getArrayFeaturesFrequency();

        // Takes all features and adds them or increases their frequencies.
        for(int i = 0; i < features_array.size(); i++) {
            //Gets a feature and its freqeuncy.
            FeatureInterface f = features_array.get(i);

            //Determine, if a feature is present in some other document
            FeatureDF fdf = this.features_document_frequency.get(f.getFeature());

            if(fdf == null) {
                this.features_document_frequency.put(f.getFeature(), new FeatureDF(f.getFeature(), FIRST_PRESENCE));
            }
            else {
                //Increase the occurence in all documents
                fdf.document_frequent = fdf.document_frequent + 1;
            }
        }
    }


    /**
     * Method returns one document, which is next to the actual cursor.
     * @return one document representation
     */
    public DocRepresentTable getDocumentSingly() {
        //1st usage of cursor
        if(cursor <= NOT_INIT) {
            cursor = START; //to 1st position
            return documents.get(cursor++);
        }
        //If it's end, returns null, the next document otherwise
        else if(cursor < documents.capacity()) {
            return documents.get(cursor++);
        }
        else {
            cursor = NOT_INIT;
            return null; 
        }
    }


    /**
     * Returns one document representation at the given position
     * @param index a position of a required document
     * @return one document representation
     */
    public DocRepresentTable getDocumentAt(int index) {
        return documents.get(index);
    }


    /**
     * Returs the count of document representations in database.
     * @return count of all documents
     */
    public int getCountDocuments() {
        return documents.size();
    }


    /**
     * Returns an IDF value of a feature for the whole dataset
     * @param feature a feature (word)
     * @return IDF value
     */
    public int getFeatureFreqInCorpus(String feature) {
        FeatureDF fdf = this.features_document_frequency.get(feature);

        if(fdf == null) {
            return 0;
        }
        else {
            return fdf.document_frequent;
        }
    }

    /**
     * Returns an array of document frequencies for all features
     * @return array of document frequencies
     */
    public ArrayList<FeatureDF> getFeaturesDocumentFreq() {
        ArrayList<FeatureDF> array = new ArrayList<FeatureDF>();
        array.addAll(this.features_document_frequency.values());
        return array;
    }

    /**
     * Returns a feature type.
     * @return feature type
     */
    public RepresentationFeature getFeature() {
        return feature;
    }

    /**
     * Returns actual representation model.
     * @return a model
     */
    public RepresentationModel getModel() {
        return model;
    }

    /**
     * Returns actual pre-processing options.
     * @return object representing pre-processing options
     */
    public Preprocessing getPreprocessing() {
        return preprocessing;
    }


    //************************************************************************//
    //                 GETTERS AND SETTERS FOR STATISTIC VARIABLES            //
    //************************************************************************//
    /**
     * Gets the average count of features in documents. 
     * @return average count of features
     */
    public double getAvgCountFeatures() {
        return avg_count_features;
    }

    /**
     * Sets the average count of features in documents.
     * @param avg_count_features average count of features
     */
    public void setAvgCountFeatures(double avg_count_features) {
        this.avg_count_features = avg_count_features;
    }

    /**
     * Gets the average feature frequency in documents.
     * @return average feature frequency value
     */
    public double getAvgFrequentFeature() {
        return avg_frequent_feature;
    }

    /**
     * Sets the average feature frequency in documents.
     * @param avg_frequent_feature average feature frequency value
     */
    public void setAvgFrequentFeature(double avg_frequent_feature) {
        this.avg_frequent_feature = avg_frequent_feature;
    }

    /**
     * Gets the count of all features in documents.
     * @return count of all features
     */
    public int getCountFeatures() {
        return count_features;
    }

    /**
     * Sets the count of all features in documents.
     * @param count_features count of all features
     */
    public void setCountFeatures(int count_features) {
        this.count_features = count_features;
    }

    /**
     * Gets the maximum count of features in one document.
     * @return maximum count of features
     */
    public int getMaxCountFeatures() {
        return max_count_features;
    }

    /**
     * Sets the maximum count of features in one document.
     * @param max_count_features maximum count of features
     */
    public void setMaxCountFeatures(int max_count_features) {
        this.max_count_features = max_count_features;
    }

    /**
     * Gets the maximum frequency of a feature among all documents.
     * @return maximum feature frequency
     */
    public int getMaxFrequentFeature() {
        return max_frequent_feature;
    }

    /**
     * Sets the maximum frequency of a feature among all documents.
     * @param max_frequent_feature maximum feature frequency
     */
    public void setMaxFrequentFeature(int max_frequent_feature) {
        this.max_frequent_feature = max_frequent_feature;
    }

    /**
     * Gets the average value of term frequency among all documents.
     * @return average value of TF
     */
    public double getAvgTFFeature() {
        return avg_tf_feature;
    }

    /**
     * Sets the average value of term frequency among all documents.
     * @param avg_tf_feature average value of TF
     */
    public void setAvgTFFeature(double avg_tf_feature) {
        this.avg_tf_feature = avg_tf_feature;
    }

    /**
     * Gets the average value of TF-IDF weight among all documents.
     * @return average value of TF-IDF
     */
    public double getAvgTFidfFeature() {
        return avg_tfidf_feature;
    }

    /**
     * Sets the average value of TF-IDF weight among all documents.
     * @param avg_tfidf_feature average value of TF-IDF
     */
    public void setAvgTFidfFeature(double avg_tfidf_feature) {
        this.avg_tfidf_feature = avg_tfidf_feature;
    }

    /**
     * Gets the maximum value of term frequency among all documents.
     * @return maximum value of TF
     */
    public double getMaxTFFeature() {
        return max_tf_feature;
    }

    /**
     * Sets the maximum value of term frequency among all documents.
     * @param max_tf_feature maximum value of TF
     */
    public void setMaxTFFeature(double max_tf_feature) {
        this.max_tf_feature = max_tf_feature;
    }

    /**
     * Gets the maximum value of TF-DF weight among all documents.
     * @return maximum value of TF-IDF
     */
    public double getMaxTFidfFeature() {
        return max_tfidf_feature;
    }

    /**
     * Sets the maximum value of TF-DF weight among all documents.
     * @param max_tfidf_feature maximum value of TF-IDF
     */
    public void setMaxTFidfFeature(double max_tfidf_feature) {
        this.max_tfidf_feature = max_tfidf_feature;
    }

    /**
     * Gets the minimum N-gram occurence in documents.
     * @return minimum N-gram occurence
     */
    public int getMinNGramOccur() {
        return min_ngram_occur;
    }

    /**
     * Sets the minimum N-gram occurence in documents.
     * @param min_ngram_occur minimum N-gram occurence
     */
    public void setMinNGramOccur(int min_ngram_occur) {
        this.min_ngram_occur = min_ngram_occur;
    }

     /**
     * Gets the N-gram depth setting (number of words in one N-gram).
     * @return N-gram depth
     */
    public int getNGramDepth() {
        return ngram_depth;
    }

     /**
     * Sets the N-gram depth setting (number of words in one N-gram).
     * @param ngram_depth N-gram depth
     */
    public void setNGramDepth(int ngram_depth) {
        this.ngram_depth = ngram_depth;
    }




}
